@@ -7,7 +7,7 @@ import textwrap
 import unittest
 
 from exif import Image
-from exif.tests.get_file_baselines import MODIFIED_NOISE_FILE_HEX_BASELINE
+from exif.tests.get_file_baselines import GRAND_CANYON_THUMBNAIL, MODIFIED_NOISE_FILE_HEX_BASELINE
 
 
 class TestGetFile(unittest.TestCase):
@@ -33,3 +33,12 @@ class TestGetFile(unittest.TestCase):
         if sys.version_info[0] == 3:
             file_hex = file_hex.decode("utf8")
         self.assertEqual('\n'.join(textwrap.wrap(file_hex, 90)), MODIFIED_NOISE_FILE_HEX_BASELINE)
+
+
+def test_get_thumbnail():
+    """Test file contents of thumbnail image."""
+    with open(os.path.join(os.path.dirname(__file__), 'grand_canyon.jpg'), 'rb') as image_file:
+        image = Image(image_file)
+
+    file_hex = binascii.hexlify(image.get_thumbnail()).decode("utf8")
+    assert '\n'.join(textwrap.wrap(file_hex, 90)) == GRAND_CANYON_THUMBNAIL
